import { App } from '@kevisual/router';
import { loadAppInfo, AppInfoConfig, saveAppInfo } from '../lib/app-file.ts';
import { fork } from 'child_process';

export enum AppType {
  SystemApp = 'system-app',
  MicroApp = 'micro-app',
  GatewayApp = 'gateway-app',
}
export type AppInfo = {
  status?: 'inactive' | 'runing' | 'stop' | 'error'; // 运行状态
  key: string;
  type?: AppType; // 默认类型
  entry?: string; // 入口文件
  path?: string; // 文件路径
  timestamp?: number; // 时间戳, 每次更新更新时间戳
  process?: any; // 进程
};
type managerOptions = {
  mainApp: App;
};
export class Manager<T extends AppInfo = any> {
  apps: Map<string, T>;
  mainApp: App;
  appInfo: AppInfoConfig;
  constructor(opts: managerOptions) {
    this.apps = new Map();
    this.mainApp = opts.mainApp;
    this.appInfo = {} as any;
  }
  /**
   * 检查key是否存在
   * @param key
   * @returns
   */
  checkKey(key: string) {
    return this.apps.has(key);
  }
  /*
   * 获取app信息
   * @param key
   */
  async loadApp(app: T) {
    const mainApp = this.mainApp;
    this.apps.set(app.key, app);
    if (app.status === 'inactive') {
      return;
    }
    const entry = app.entry + `?timestamp=${app?.timestamp}`;
    // 注册路由
    if (app.type === AppType.MicroApp) {
      const process = fork(entry, [], {});
      app.process = process;
    } else if (app.type === AppType.SystemApp) {
      const module = await import(entry);
      if (module.loadApp) {
        await module.loadApp?.(mainApp);
      }
    } else if (app.type === AppType.GatewayApp) {
      console.log('gateway app not support');
    }
  }
  /**
   * create new app info
   * @param app
   */
  async saveAppInfo(app: T, newTimeData = false) {
    const list = this.appInfo.list || [];
    if (newTimeData) {
      app.timestamp = Date.now();
    }
    const { process, ...info } = app;
    list.push(info);
    this.appInfo.list = list;
    await saveAppInfo(this.appInfo);
  }

  /**
   * 初始化应用的时候加载
   */
  async load() {
    // 从apps文件夹列表当中中加载app信息
    const appInfos = await loadAppInfo();
    const list = appInfos?.list || [];
    for (const app of list) {
      try {
        this.loadApp(app);
      } catch (e) {
        console.error('load app', e);
      }
    }
  }
  async add(app: T) {
    if (this.checkKey(app.key)) {
      return false;
    }

    this.loadApp(app);
    await this.saveAppInfo(app, true);
  }
  // 启动
  async start(key: string) {
    const app = this.apps.get(key);
    if (!app) {
      return;
    }
    if (app.status === 'runing') {
      return;
    }
    app.status = 'runing';
    this.loadApp(app);
    await this.saveAppInfo(app);
  }
  // 停止
  async stop(key: string) {
    const app = this.apps.get(key);
    if (!app) {
      return;
    }
    if (app.status === 'stop') {
      return;
    }
    app.status = 'stop';
    if (app.process) {
      app.process.kill();
    }
    await this.saveAppInfo(app);
  }
}
